@@ -11,8 +11,9 @@ has 'attrName' =>
 	;
 
 method node_ok( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
-	return ( $node->nodeType == XML_TEXT_NODE or
-			 $node->nodeType == XML_CDATA_SECTION_NODE );
+	( $node->nodeType == XML_TEXT_NODE or
+		  $node->nodeType == XML_CDATA_SECTION_NODE )
+		? 1 : undef;
 }
 
 method accept( XML::LibXML::Node $node, PRANG::Graph::Context $ctx ) {
